<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 12/16/2008 *
 * ===================                          Last change: 12/16/2008 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : filters.php                                      *
 * -------------------------------------------------------------------- *
 * Short description : Generic filters                                  *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Allgemeine Filter                                *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * Needs to be in all Files and every File needs "svn propset           *
 * svn:keywords Date Revision" (autoprobset!) at least!!!!!!            *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Filter for flushing all new filters to the database
function FILTER_FLUSH_FILTERS () {
	// Clear all previous SQL queries
	initSqls();

	// Are we installing?
	if ((isInstallationPhase())) {
		// Then silently skip this filter
		return true;
	} // END - if

	// Is a database link here and not in installation mode?
	if ((!SQL_IS_LINK_UP()) && (!isInstalling())) {
		// Abort here
		addFatalMessage(__FUNCTION__, __LINE__, getMessage('FILTER_FLUSH_FAILED_NO_DATABASE'));
		return false;
	} // END - if

	// Is the extension sql_patches updated?
	if ((!isExtensionInstalled('sql_patches')) || (isExtensionInstalledAndOlder('sql_patches', '0.5.9'))) {
		// Abort silently here
		return false;
	} // END - if

	// Nothing is added/remove by default
	$inserted = '0';
	$removed = '0';

	// Prepare SQL queries
	$insertSQL = "INSERT INTO `{?_MYSQL_PREFIX?}_filters` (`filter_name`,`filter_function`,`filter_active`) VALUES";
	$removeSQL = "DELETE LOW_PRIORITY FROM `{?_MYSQL_PREFIX?}_filters` WHERE";

	// Write all filters to database
	foreach ($GLOBALS['cache_array']['filter']['chains'] as $filterName => $filterArray) {
		// Walk through all filters
		foreach ($filterArray as $filterFunction => $active) {
			// Is this filter loaded?
			//* DEBUG: */ print 'FOUND:'.$filterName.'/'.$filterFunction.'='.$active.'<br />';
			if (((!isset($GLOBALS['cache_array']['filter']['loaded'][$filterName][$filterFunction])) && ($active != 'R')) || ($active == 'A')) {
				// Add this filter (all filters are active by default)
				//* DEBUG: */ print 'ADD:'.$filterName.'/'.$filterFunction.'<br />';
				$insertSQL .= sprintf("('%s','%s','Y'),", $filterName, $filterFunction);
				$inserted++;
			} elseif ($active == 'R') {
				// Remove this filter
				//* DEBUG: */ print 'REMOVE:'.$filterName.'/'.$filterFunction.'<br />';
				$removeSQL .= sprintf(" (`filter_name`='%s' AND `filter_function`='%s') OR", $filterName, $filterFunction);
				$removed++;
			}
		} // END - foreach
	} // END - foreach

	// Something has been added?
	if ($inserted > 0) {
		// Finish SQL command and add it
		addSql(substr($insertSQL, 0, -1));
	} // END - if

	// Something has been removed?
	if ($removed > 0) {
		// Finish SQL command and add it
		addSql(substr($removeSQL, 0, -2) . 'LIMIT ' . $removed);
	} // END - if

	// Shall we update usage counters (ONLY FOR DEBUGGING!)
	if (getConfig('update_filter_usage') == 'Y') {
		// Update all counters
		foreach ($GLOBALS['cache_array']['filter']['counter'] as $filterName => $filterArray) {
			// Walk through all filters
			foreach ($filterArray as $filterFunction => $cnt) {
				// Construct and add the query
				addSql(sprintf("UPDATE `{?_MYSQL_PREFIX?}_filters` SET `filter_counter`=%s WHERE `filter_name`='%s' AND `filter_function`='%s' LIMIT 1",
					bigintval($cnt),
					$filterName,
					$filterFunction
				));
			} // END - foreach
		} // END - foreach
	} // END - if

	// Run the run_sqls filter in non-dry mode
	runFilterChain('run_sqls');

	// Should we rebuild cache?
	if (($inserted > 0) || ($removed > 0)) {
		// Destroy cache
		rebuildCacheFile('filter', 'filter');
	} // END - if
}

// Filter for calling the handler for login failures
function FILTER_CALL_HANDLER_LOGIN_FAILTURES ($data) {
	// Init content
	$content = $data;

	// Handle failed logins here if not in guest
	//* DEBUG: */ outputHtml(__FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):type={$data['type']},action={getAction()},what={getWhat()},level={$data['access_level']}<br />");
	if ((($data['type'] == 'what') || ($data['type'] == 'action') && ((!isWhatSet()) || (getWhat() == 'overview') || (getWhat() == getConfig('index_home')))) && ($data['access_level'] != 'guest') && ((isExtensionInstalledAndNewer('sql_patches', '0.4.7')) || (isExtensionInstalledAndNewer('admins', '0.7.0')))) {
		// Handle failure
		$content['content'] .= handleLoginFailtures($data['access_level']);
	} // END - if

	// Return the content
	return $content;
}

// Filter for redirecting to logout if sql_patches has been installed
function FILTER_REDIRECT_TO_LOGOUT_SQL_PATCHES () {
	// Remove this filter
	unregisterFilter('shutdown', __FUNCTION__);

	// Is the element set?
	if (isset($GLOBALS['ext_load_mode'])) {
		// Redirect here
		redirectToUrl('modules.php?module=admin&amp;logout=1&amp;' . $GLOBALS['ext_load_mode'] . '=sql_patches');
	} // END - if

	// This should not happen!
	logDebugMessage(__FUNCTION__, __LINE__, 'Cannot auto-logout because no extension load-mode has been set.');
}

// Filter for auto-activation of a extension
function FILTER_AUTO_ACTIVATE_EXTENSION ($data) {
	// Is this extension always activated?
	if (getExtensionAlwaysActive() == 'Y') {
		// Then activate the extension
		//* DEBUG: */ outputHtml(__FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>): ext_name={$data['ext_name']}<br />");
		doActivateExtension($data['ext_name']);
	} // END - if

	// Return the data
	return $data;
}

// Filter for solving task given task
function FILTER_SOLVE_TASK ($data) {
	// Don't solve anything if no admin!
	if (!isAdmin()) return $data;

	// Is this a direct task id or array element task_id is found?
	if (is_int($data)) {
		// Then solve it...
		adminSolveTask($data);
	} elseif ((is_array($data)) && (isset($data['task_id']))) {
		// Solve it...
		adminSolveTask($data['task_id']);
	} else {
		// Not detectable!
		debug_report_bug(sprintf("Cannot resolve task. data[%s]=<pre>%s</pre>", gettype($data), print_r($data, true)));
	}

	// Return the data
	return $data;
}

// Filter to load include files
function FILTER_LOAD_INCLUDES ($pool) {
	// Is it null?
	if (is_null($pool)) {
		// This should not happen!
		debug_report_bug('pool is null.');
	} // END - if

	// Is the pool an array and 'pool' set?
	if ((is_array($pool)) && (isset($pool['pool']))) {
		// Then use it as pool
		$realPool = $pool['pool'];
	} else {
		// Default is $data as inclusion list
		$realPool = $pool;
	}

	// Get inc pool
	$data = getIncludePool($realPool);

	// Is it an array?
	if ((!isset($data)) || (!is_array($data))) {
		// Then abort here
		debug_report_bug(sprintf("INC_POOL is no array! Type: %s", gettype($data)));
	} elseif (isset($data['inc_pool'])) {
		// Use this as new inclusion pool!
		setIncludePool($realPool, $data['inc_pool']);
	}

	// Check for added include files
	if (countIncludePool($realPool) > 0) {
		// Loads every include file
		loadIncludePool($realPool);

		// Reset array
		initIncludePool($realPool);
	} // END - if

	// Continue with processing
	return $pool;
}

// Filter for running SQL commands
function FILTER_RUN_SQLS ($data) {
	// Debug message
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, "- Entered!");

	// Is the array there?
	if ((isSqlsValid()) && ((!isset($data['dry_run'])) || ($data['dry_run'] == false))) {
		// Run SQL commands
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, "- Found ".countSqls()." queries to run.");
		foreach (getSqls() as $sqls) {
			// New cache format...
			foreach ($sqls as $sql) {
				// Trim spaces away
				$sql = trim($sql);

				// Is there still a query left?
				if (!empty($sql)) {
					// Do we have an "ALTER TABLE" command?
					if (substr(strtolower($sql), 0, 11) == 'alter table') {
						// Analyse the alteration command
						//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, "Alterting table: {$sql}");
						SQL_ALTER_TABLE($sql, __FUNCTION__, __LINE__);
					} else {
						// Run regular SQL command
						//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, "Running regular query: {$sql}");
						SQL_QUERY($sql, __FUNCTION__, __LINE__, false);
					}
				} // END - if
			} // END - foreach
		} // END - foreach
	} // END - if

	// Debug message
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, "- Left!");
}

// Filter for updating/validating login data
function FILTER_UPDATE_LOGIN_DATA () {
	// Add missing array
	if ((!isset($GLOBALS['last_online'])) || (!is_array($GLOBALS['last_online']))) $GLOBALS['last_online'] = array();

	// Recheck if logged in
	if (!isMember()) return false;

	// Secure user id
	setMemberId(getSession('userid'));

	// Found a userid?
	if (fetchUserData(getMemberId())) {
		// Load last module and online time
		$content = getUserDataArray();

		// Maybe first login time?
		if (empty($content['last_module'])) $content['last_module'] = 'login';

		// This will be displayed on welcome page! :-)
		if (empty($GLOBALS['last_online']['module'])) {
			$GLOBALS['last_online']['module'] = $content['last_module'];
			$GLOBALS['last_online']['online'] = $content['last_online'];
		} // END - if

		// 'what' not set?
		if (!isWhatSet()) {
			// Fix it to default
			setWhat('welcome');
			if (getConfig('index_home') != '') setWhatFromConfig('index_home');
		} // END - if

		// Update last module / online time
		updateLastActivity(getMemberId());
	}  else {
		// Destroy session, we cannot update!
		destroyMemberSession();
	}
}

// Filter for initializing randomizer
function FILTER_INIT_RANDOMIZER () {
	// Only execute this filter if installed
	if ((!isInstalled()) || (!isExtensionInstalledAndNewer('other', '0.2.5'))) return;

	// Take a prime number which is long (if you know a longer one please try it out!)
	setConfigEntry('_PRIME', 591623);

	// Calculate "entropy" with the prime number (for code generation)
	setConfigEntry('_ADD', (getConfig('_PRIME') * getConfig('_PRIME') / (pi() * getConfig('code_length') + 1)));

	// Simply init the randomizer with seed and _ADD value
	mt_srand(generateSeed() + getConfig('_ADD'));
}

// Filter for removing updates
function FILTER_REMOVE_UPDATES ($data) {
	// Init removal list
	initExtensionRemovalList();

	// Add the current extension to it
	addCurrentExtensionToRemovalList();

	// Simply remove it
	unsetExtensionSqls();

	// Do we need to remove update depency?
	if (countExtensionUpdateDependencies() > 0) {
		// Then find all updates we shall no longer execute
		foreach (getExtensionUpdateDependencies() as $id => $ext_name) {
			// Shall we remove this update?
			if (in_array($ext_name, getExtensionRemovalList())) {
				// Then remove this extension!
				removeExtensionUpdateDependency($ext_name);
			} // END - if
		} // END - foreach
	} // END - if

	// Return data
	return $data;
}

// Determines username for current user state
function FILTER_DETERMINE_USERNAME () {
	// Check if logged in
	if (isMember()) {
		// Is still logged in so we welcome him with his name
		if (fetchUserData(getMemberId())) {
			// Load surname and family's name and build the username
			$content = getUserDataArray();

			// Prepare username
			setUsername($content['surname'] . ' ' . $content['family']);

			// Additionally admin?
			if (isAdmin()) {
				// Add it
				setUsername(getUsername() . ' ({--USERNAME_ADMIN_SHORT--})');
			} // END - if
		} else {
			// Hmmm, logged in and no valid userid?
			setUsername('<em>{--USERNAME_UNKNOWN--}</em>');

			// Destroy session
			destroyMemberSession();
		}
	} elseif (isAdmin()) {
		// Admin is there
		setUsername('{--USERNAME_ADMIN--}');
	} else {
		// He's a guest, hello there... ;-)
		setUsername('{--USERNAME_GUEST--}');
	}
}

// Filter for compiling config entries
function FILTER_COMPILE_CONFIG ($code, $compiled = false) {
	// Save the uncompiled code
	$uncompiled = $code;

	// Do we have cache?
	if (!isset($GLOBALS['compiled_config'][$code])) {
		// Compile {?some_var?} to getConfig('some_var')
		preg_match_all('/\{\?(([a-zA-Z0-9-_]+)*)\?\}/', $code, $matches);

		// Some entries found?
		if ((count($matches) > 0) && (count($matches[0]) > 0)) {
			// Replace all matches
			foreach ($matches[0] as $key => $match) {
				// Do we have cache?
				if (!isset($GLOBALS['compile_config'][$matches[1][$key]])) {
					// Is the config valid?
					if (isConfigEntrySet($matches[1][$key])) {
						// Set it for caching
						$GLOBALS['compile_config'][$matches[1][$key]] = "\".getConfig('" . $matches[1][$key] . "').\"";
					} elseif (isConfigEntrySet('default_' . strtoupper($matches[1][$key]))) {
						// Use default value
						$GLOBALS['compile_config'][$matches[1][$key]] = "\".getConfig('" . 'DEFAULT_' . strtoupper($matches[1][$key]) . "').\"";
					} elseif (isMessageIdValid('DEFAULT_' . strtoupper($matches[1][$key]))) {
						// No config, try the language system
						$GLOBALS['compile_config'][$matches[1][$key]] = "\".getMessage('". 'DEFAULT_' . strtoupper($matches[1][$key]) . "').\"";
					} else {
						// Unhandled!
						$GLOBALS['compile_config'][$matches[1][$key]] = '!' . $matches[1][$key] . '!';
					}
				} // END - if

				// Use this for replacing
				$code = str_replace($match, $GLOBALS['compile_config'][$matches[1][$key]], $code);
				//* DEBUG: */ if (($match == '{?URL?}') && (strlen($code) > 10000)) die('<pre>'.secureString($code).'</pre>');
			} // END - foreach
		} // END - if

		// Add it to cache
		$GLOBALS['compiled_config'][$uncompiled] = $code;
	} // END - if

	// Should we compile it?
	if ($compiled === true) {
		// Run the code
		$eval = "\$GLOBALS['compiled_config'][\$uncompiled] = \"" . $GLOBALS['compiled_config'][$uncompiled] . "\";";
		//* DEBUG: */ print '<pre>' . str_replace('$', '&#36;', htmlentities($eval)) . '</pre>';
		eval($eval);
	} // END - if

	// Return compiled code
	return $GLOBALS['compiled_config'][$uncompiled];
}

// Filter for compiling extension data
function FILTER_COMPILE_EXTENSION ($code) {
	// Compile {%cmd=some_value%} to get extension data
	// Support cmd is:
	//   - version -> getExtensionVersion() call
	preg_match_all('/\{%((([a-zA-Z0-9-_]+)=([a-zA-Z0-9-_]+))*)\%\}/', $code, $matches);

	// Some entries found?
	if ((count($matches) > 0) && (count($matches[3]) > 0)) {
		// Replace all matches
		foreach ($matches[3] as $key => $cmd) {
			// Init replacer variable
			$replacer = '';

			// Is the extension installed or code provided?
			if ($cmd == 'code') {
				// Code asked for
				$replacer = "\".getCode(\"" . $matches[4][$key] . "\").\"";
			} else {
				// Construct call-back function name
				$functionName = 'getExtension' . ucfirst(strtolower($cmd));

				// Construct call of the function
				$replacer = "\".call_user_func_array('" . $functionName . "', array('" . $matches[4][$key] . "', true)).\"";
			}

			// Replace it and insert parameter for GET request
			$code = str_replace($matches[0][$key], sprintf("&amp;%s=%s&amp;rev=\".getConfig('CURR_SVN_REVISION').\"", $cmd, $replacer), $code);
		} // END - foreach
	} // END - if

	// Return compiled code
	//* DEBUG: */ die('<pre>'.secureString($code).'</pre>');
	return $code;
}

// Runs some generic filter update steps
function FILTER_UPDATE_EXTENSION_DATA ($ext_name) {
	// Create task
	createExtensionUpdateTask(getCurrentAdminId(), $ext_name, $GLOBALS['update_ver'][$ext_name], SQL_ESCAPE(getExtensionNotes(getExtensionNotes())));

	// Update extension's version
	SQL_QUERY_ESC("UPDATE `{?_MYSQL_PREFIX?}_extensions` SET `ext_version`='%s' WHERE `ext_name`='%s' LIMIT 1",
		array($GLOBALS['update_ver'][$ext_name], $ext_name), __FUNCTION__, __LINE__);

	// Remove arrays
	unsetSqls();
	unset($GLOBALS['update_ver'][$ext_name]);
}

// Load more reset scripts
function FILTER_RUN_RESET_INCLUDES () {
	// Is the reset set or old sql_patches?
	if (((!isResetModeEnabled()) || (!isExtensionInstalled('sql_patches'))) && (getOutputMode() == '0')) {
		// Then abort here
		logDebugMessage(__FUNCTION__, __LINE__, 'Cannot run reset! Please report this bug. Thanks');
	} // END - if

	// Get more daily reset scripts
	setIncludePool('reset', getArrayFromDirectory('inc/reset/', 'reset_'));

	// Update database
	if ((!isConfigEntrySet('DEBUG_RESET')) || (getConfig('DEBUG_RESET') != 'Y')) updateConfiguration('last_update', 'UNIX_TIMESTAMP()');

	// Is the config entry set?
	if (isExtensionInstalledAndNewer('sql_patches', '0.4.2')) {
		// Create current week mark
		$currWeek = date('W', time());

		// Has it changed?
		if ((getConfig('last_week') != $currWeek) || (getConfig('DEBUG_WEEKLY') == 'Y')) {
			// Include weekly reset scripts
			mergeIncludePool('reset', getArrayFromDirectory('inc/weekly/', 'weekly_'));

			// Update config
			if ((isConfigEntrySet('DEBUG_WEEKLY')) && (getConfig('DEBUG_WEEKLY') != 'Y')) updateConfiguration('last_week', $currWeek);
		} // END - if

		// Create current month mark
		$currMonth = date('m', time());

		// Has it changed?
		if ((getConfig('last_month') != $currMonth) || (getConfig('DEBUG_MONTHLY') == 'Y')) {
			// Include monthly reset scripts
			mergeIncludePool('reset', getArrayFromDirectory('inc/monthly/', 'monthly_'));

			// Update config
			if ((isConfigEntrySet('DEBUG_MONTHLY')) && (getConfig('DEBUG_MONTHLY') != 'Y')) updateConfiguration('last_month', $currMonth);
		} // END - if
	} // END - if

	// Run the filter
	runFilterChain('load_includes', 'reset');
}

// Filter for removing the given extension
function FILTER_REMOVE_EXTENSION () {
	// Delete this extension (remember to remove it from your server *before* you click on welcome!
	SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM `{?_MYSQL_PREFIX?}_extensions` WHERE `ext_name`='%s' LIMIT 1",
		array(getCurrentExtensionName()), __FUNCTION__, __LINE__);

	// Remove the extension from global cache array as well
	removeExtensionFromArray();

	// Remove the cache
	rebuildCacheFile('extension', 'extension');
}

// Filter for flushing the output
function FILTER_FLUSH_OUTPUT () {
	// Simple, he?
	outputHtml('');
}

// Prepares an SQL statement part for HTML mail and/or holiday depency
function FILTER_HTML_INCLUDE_USERS ($mode) {
	// Exclude no users by default
	$MORE = '';

	// HTML mail?
	if ($mode == 'html') $MORE = " AND `html`='Y'";
	if ((isExtensionActive('holiday')) && (getExtensionVersion('holiday') >= '0.1.3')) {
		// Add something for the holiday extension
		$MORE .= " AND `holiday_active`='N'";
	} // END - if

	// Return result
	return $MORE;
}

// Filter for determining what/action/module
function FILTER_DETERMINE_WHAT_ACTION () {
	// In installation phase we don't have what/action
	if (isInstallationPhase()) {
		// Set both to empty
		setAction('');
		setWhat('');

		// Abort here
		return;
	} // END - if

	// Get all values
	if ((getOutputMode() != 1) && (getOutputMode() != -1)) {
		// Fix module
		if (!isModuleSet()) {
			// Is the request element set?
			if (isGetRequestElementSet('module')) {
				// Set module from request
				setModule(getRequestElement('module'));
			} elseif (getOutputMode() == '0') {
				// Set default module 'index'
				setModule('index');
			} else {
				// Unknown module
				setModule('unknown');
			}
		} // END - if

		// Fix 'what' if not yet set
		if (!isWhatSet())   setWhat(getWhatFromModule(getModule()));

		// Fix 'action' if not yet set
		if (!isActionSet()) setAction(getModeAction(getModule(), getWhat()));
	} else {
		// Set action/what to empty
		setAction('');
		setWhat('');
	}

	// Set default 'what' value
	//* DEBUG: */ outputHtml('-'.getModule().'/'.getWhat()."-<br />");
	if ((!isWhatSet()) && (!isActionSet()) && (getOutputMode() != 1) && (getOutputMode() != -1)) {
		if (getModule() == 'admin') {
			// Set 'action' value to 'login' in admin menu
			setAction(getModeAction(getModule(), getWhat()));
		} elseif ((getModule() == 'index') || (getModule() == 'login')) {
			// Set 'what' value to 'welcome' in guest and member menu
			setWhatFromConfig('index_home');
		} else {
			// Anything else like begging link
			setWhat('');
		}
	} // END - if
}

// Sends out pooled mails
function FILTER_TRIGGER_SENDING_POOL () {
	// Are we in normal output mode?
	if (getOutputMode() != 0) {
		// Only in normal output mode to prevent race-conditons!
	} // END - if

	// Init counter
	$GLOBALS['pool_cnt'] = '0';

	// Init & set the include pool
	initIncludePool('pool');
	setIncludePool('pool', getArrayFromDirectory('inc/pool/', 'pool-'));

	// Run the filter
	runFilterChain('load_includes', 'pool');

	// Remove the counter
	unset($GLOBALS['pool_cnt']);
}

// Filter for checking and updating SVN revision
function FILTER_CHECK_SVN_REVISION () {
	// Only execute this filter if installed and all config entries are there
	if ((!isInstalled()) || (!isConfigEntrySet('patch_level'))) return;

	// Check for patch level differences between databases and current hard-coded
	if ((getConfig('CURR_SVN_REVISION') > getConfig('patch_level')) || (getConfig('patch_level') == 'CURR_SVN_REVISION') || (getConfig('patch_ctime') == 'UNIX_TIMES')) {
		// Update database and CONFIG array
		updateConfiguration(array('patch_level', 'patch_ctime'), array(getConfig('CURR_SVN_REVISION'), 'UNIX_TIMESTAMP()'));
		setConfigEntry('patch_level', getConfig('CURR_SVN_REVISION'));
		setConfigEntry('patch_ctime', time());
	} // END - if
}

// Filter for running daily reset
function FILTER_RUN_DAILY_RESET () {
	// Only execute this filter if installed
	if ((!isInstalled()) || (!isAdminRegistered())) return;

	// Shall we run the reset scripts? If a day has changed, maybe also a week/month has changed... Simple! :D
	// 012    3              4             43        3         4432    2         3             3       21    1                    221    1                 221    1                  2          21    1             22     10
	if (((date('d', getConfig('last_update')) != date('d', time())) || ((isConfigEntrySet('DEBUG_RESET')) && (getConfig('DEBUG_RESET') == 'Y'))) && (!isInstallationPhase()) && (isAdminRegistered()) && (!isGetRequestElementSet('register')) && (getOutputMode() != 1)) {
		// Tell every module we are in reset-mode!
		doReset();
	} // END - if
}

// Filter for loading more runtime includes (not for installation)
function FILTER_LOAD_RUNTIME_INCLUDES () {
	// Load more includes
	foreach (array('inc/databases.php','inc/session.php','inc/versions.php') as $inc) {
		// Load the include
		loadIncludeOnce($inc);
	} // END - foreach

	// Load admin include file if he is admin
	if (isAdmin()) {
		// Administrative functions
		loadIncludeOnce('inc/modules/admin/admin-inc.php');
	} // END - if
	//* DEBUG: */ addPointsThroughReferalSystem('test', 36, 1000);
	//* DEBUG: */ die();
}

// Filter for checking admin ACL
function FILTER_CHECK_ADMIN_ACL () {
	// Extension not installed so it's always allowed to access everywhere!
	$ret = true;

	// Ok, Cookie-Update done
	if ((isExtensionInstalledAndNewer('admins', '0.3.0')) && (isExtensionActive('admins'))) {
		// Check if action GET variable was set
		$action = getAction();
		if (isWhatSet()) {
			// Get action value by what-value
			$action = getModeAction('admin', getWhat());
		} // END - if

		// Check for access control line of current menu entry
		$ret = adminsCheckAdminAcl($action, getWhat());
	} // END - if

	// Set it here
	$GLOBALS['acl_allow'] = $ret;
}

// Init random number/cache buster
function FILTER_INIT_RANDOM_NUMBER () {
	// Is the extension sql_patches installed and at least 0.3.6?
	if ((isExtensionInstalledAndNewer('sql_patches', '0.3.6')) && (isExtensionInstalledAndNewer('other', '0.2.5'))) {
		// Generate random number
		setConfigEntry('RAND_NUMBER', generateRandomCode(10, mt_rand(10000, 32766), getMemberId(), ''));
	} else {
		// Generate weak (!!!) code
		setConfigEntry('RAND_NUMBER', mt_rand(1000000, 9999999));
	}

	// Copy it to CACHE_BUSTER
	setConfigEntry('CACHE_BUSTER', getConfig('RAND_NUMBER'));
}

// Update module counter
function FILTER_COUNT_MODULE () {
	// Do count all other modules but not accesses on CSS file css.php!
	SQL_QUERY_ESC("UPDATE `{?_MYSQL_PREFIX?}_mod_reg` SET `clicks`=`clicks`+1 WHERE `module`='%s' LIMIT 1",
		array(getModule()), __FUNCTION__, __LINE__);
}

// Handles fatal errors
function FILTER_HANDLE_FATAL_ERRORS () {
	// Do we have errors to handle and right output mode?
	if ((getTotalFatalErrors() == '0') || (getOutputMode() != 0)) {
		// Abort executing here
		return false;
	} // END - if

	// Set content type
	setContentType('text/html');

	// Load config here
	loadIncludeOnce('inc/load_config.php');

	// Set unset variable
	if (empty($check)) $check = '';

	// Default is none
	$content = '';

	// Installation phase or regular mode?
	if ((isInstallationPhase())) {
		// While we are installing ouput other header than while it is installed... :-)
		$OUT = '';
		foreach (getFatalArray() as $key => $value) {
			// Prepare content for the template
			$content = array(
				'key'   => ($key + 1),
				'value' => $value
			);

			// Load row template
			$OUT .= loadTemplate('install_fatal_row', true, $content);
		}

		// Load main template
		$content = loadTemplate('install_fatal_table', true, $OUT);
	} elseif (isInstalled()) {
		// Display all runtime fatal errors
		$OUT = '';
		foreach (getFatalArray() as $key => $value) {
			// Prepare content for the template
			$content = array(
				'key'   => ($key + 1),
				'value' => $value
			);

			// Load row template
			$OUT .= loadTemplate('runtime_fatal_row', true, $content);
		}

		// Load main template
		$content = loadTemplate('runtime_fatal_table', true, $OUT);
	}

	// Message to regular users (non-admin)
	$CORR = getMessage('FATAL_REPORT_ERRORS');

	// PHP warnings fixed
	if ($check == 'done') {
		if (isAdmin()) $CORR = getMessage('FATAL_CORRECT_ERRORS');
	} // END - if

	// Remember all in array
	$content = array(
		'rows' => $content,
		'corr' => $CORR
	);

	// Load footer
	loadIncludeOnce('inc/header.php');

	// Load main template
	loadTemplate('fatal_errors', false, $content);

	// Delete all to prevent double-display
	initFatalMessages();

	// Load footer
	loadIncludeOnce('inc/footer.php');

	// Abort here
	shutdown();
}

// Filter for displaying copyright line
function FILTER_DISPLAY_COPYRIGHT () {
	// Shall we display the copyright notice?
	if ((!isGetRequestElementSet('frame')) && (basename($_SERVER['PHP_SELF']) != 'mailid_top.php') && ((getConfig('WRITE_FOOTER') == 'Y') || (isInstalling())) && ($GLOBALS['header_sent'] == 2)) {
		// Backlink enabled?
		if (((isConfigEntrySet('ENABLE_BACKLINK')) && (getConfig('ENABLE_BACKLINK') == 'Y')) || (isInstalling())) {
			// Copyright with backlink, thanks! :-)
			loadTemplate('copyright_backlink');
		} else {
			// No backlink in Copyright note
			loadTemplate('copyright');
		}
	} // END - if
}

// Filter for displaying parsing time
function FILTER_DISPLAY_PARSING_TIME () {
	// Shall we display the parsing time and number of queries?
	// 1234                            5                      54    4         5              5       4    4                       5       543    3                   4432    2             33     2    2                              21
	if ((((isExtensionInstalledAndNewer('sql_patches', '0.4.1')) && (getConfig('show_timings') == 'Y') && (!isGetRequestElementSet('frame'))) || (isInstallationPhase())) && (getOutputMode() == '0') && ($GLOBALS['header_sent'] == 2)) {
		// Then display it here
		displayParsingTime();
	} // END - if
}

// Filter for flushing template cache
function FILTER_FLUSH_TEMPLATE_CACHE () {
	// Do not flush when debugging the template cache
	if (isDebuggingTemplateCache()) return;

	// Do we have cached eval() data?
	if ((isset($GLOBALS['template_eval'])) && (count($GLOBALS['template_eval']) > 0)) {
		// Now flush all
		foreach ($GLOBALS['template_eval'] as $template => $eval) {
			// Flush the cache (if not yet found)
			flushTemplateCache($template, $eval);
		} // END - if
	} // END - if
}

// [EOF]
?>
